
subroutine CalcEnergy(Pos, L, kappa, lambda, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, kappa, lambda
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 5000., r0 = 1., rc2 = 2.0 ** (1.0/3.0)
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12, sign
    integer :: i, j
    PEnergy = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            !compute only the squared distance and compare to squared cut            
            !monomer electrostatics
            sign = 1.0
            if( i < NAtom/2 .and. j >= NAtom/2) then
                sign = -1.0
            endif
            d = sqrt(sum(rij * rij))
            PEnergy = PEnergy + sign*lambda/d*exp(-1.0*d*kappa)
            !bonded pair potential
            if ( j == i + 1 .and. j /= NAtom/2) then 
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0) * (d - r0)
            !non-bonded pair potential
            else
                d2 = sum(rij * rij)                
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2             !inverse squared distance
                id6 = id2 * id2 * id2     !inverse sixth distance
                id12 = id6 * id6          !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + 1.
            endif
        enddo
    enddo
end subroutine


subroutine CalcEnergyForces(Pos, L, kappa, lambda, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, kappa, lambda
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 5000., r0 = 1., rc2 = 2.0 ** (1.0/3.0)
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d, d2, id2, id6, id12, sign
    integer :: i, j
    PEnergy = 0.
    Forces = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            !compute only the squared distance and compare to squared cut            
            !monomer electrostatics
            sign = 1.0
            if( i < NAtom/2 .and. j >= NAtom/2) then
                sign = -1.0
            endif
            d = sqrt(sum(rij * rij))
            PEnergy = PEnergy + sign*lambda/d*exp(-1.0*d*kappa)
            Fij = rij * (-1.0*sign*lambda*(1.0/(d*d*d)+kappa/(d*d))*exp(-1.0*d*kappa))
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
            !bonded pair potential
            if ( j == i + 1 .and. j /= NAtom/2) then 
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2.0 * (d - r0) * (d - r0)
                Fij = rij * k * (1-r0/d)
            !non-bonded pair potential
            else
                d2 = sum(rij * rij)                
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2             !inverse squared distance
                id6 = id2 * id2 * id2     !inverse sixth distance
                id12 = id6 * id6          !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + 1.
                Fij = rij * ((-48. * id12 + 24. * id6) * id2)
            endif
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
        enddo
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, L, kappa, lambda, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in) :: L, kappa, lambda, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, L, kappa, lambda, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine
